@@ -15,13 +15,13 @@ test_that("Total outputs", {
   o <- sample_systematic(raster = mraster, cellsize = 1000,square = FALSE, plot = TRUE)
   o1 <- sample_systematic(raster = mraster, cellsize = 1000)
   
-  expect_equal(nrow(o), 39L)
+  expect_equal(nrow(o), 43L)
   expect_equal(ncol(o), 1L)
   
   expect_equal(nrow(o1), 34L)
   expect_equal(ncol(o1), 1L)
   
-  expect_equal(nrow(sample_systematic(raster = mraster, cellsize = 2000, details = TRUE)$samples), 12L)
+  expect_equal(nrow(sample_systematic(raster = mraster, cellsize = 2000, details = TRUE)$samples), 9L)
   expect_s3_class(o,"sf")
   expect_s3_class(sample_systematic(raster = mraster, cellsize = 2000, details = TRUE)$tessellation,"sf")
 })
@@ -32,8 +32,8 @@ test_that("corners", {
   o1 <- sample_systematic(raster = mraster, cellsize = 1000, location = "corners")
   o2 <- sample_systematic(raster = mraster, cellsize = 1000, square = FALSE, location = "corners")
   
-  expect_equal(nrow(o1), 40L)
-  expect_equal(nrow(o2), 282L)
+  expect_equal(nrow(o1), 195L)
+  expect_equal(nrow(o2), 302L)
 })
 
 test_that("random", {
@@ -41,8 +41,10 @@ test_that("random", {
   or <- sample_systematic(raster = mraster, cellsize = 1000, location = "random")
   or1 <- sample_systematic(raster = mraster, cellsize = 1000, square = FALSE, location = "random")
   
-  expect_equal(nrow(or), 32L)
+  expect_equal(nrow(or), 34L)
   expect_equal(nrow(or1), 41L)
+  
+  expect_error(sample_systematic(raster = sraster, cellsize = 5000000, location = "random"),"No samples intersect with 'raster'. Ensure 'cellsize' makes sense.")
 })
 
 
@@ -57,4 +59,5 @@ test_that("messages", {
 
   expect_message(sample_systematic(raster = sraster, cellsize = 20000, square = FALSE, location = "random", force = TRUE),"Forcing samples to fall in non NA locations.")
   expect_message(sample_systematic(raster = sraster, cellsize = 5000, location = "random", force = TRUE),"Forcing samples to fall in non NA locations.")
+  expect_error(sample_systematic(raster = sraster, cellsize = 5000000, location = "centers"),"No samples intersect with 'raster'. Ensure 'cellsize' makes sense.")
 })
